@@ -13,13 +13,18 @@
 ############################################################################
 """Exported transaction functions.
 
-$Id: __init__.py 112264 2010-05-12 20:36:53Z jim $
+$Id: __init__.py 123062 2011-10-12 10:05:08Z chrism $
 """
 
 from transaction._transaction import Transaction
 from transaction._manager import TransactionManager
 from transaction._manager import ThreadTransactionManager
 
+# NB: "with transaction:" does not work under Python 3 because they worked
+# really hard to break looking up special methods like __enter__ and __exit__
+# via getattr and getattribute; see http://bugs.python.org/issue12022.  On
+# Python 3, you must use ``with transaction.manager`` instead.
+
 manager = ThreadTransactionManager()
 get = __enter__ = manager.get
 begin = manager.begin
