@@ -1,6 +1,6 @@
 ############################################################################
 #
-# Copyright (c) 2004 Zope Corporation and Contributors.
+# Copyright (c) 2004 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -55,8 +55,8 @@ arguments at the start of the commit (but not for substransaction commits).
 After-commit hook
 ------------------
 
-Sometimes, applications want to execute code after a transaction is
-committed or aborted. For example, one might want to launch non
+Sometimes, applications want to execute code after a transaction commit
+attempt succeeds or aborts. For example, one might want to launch non
 transactional code after a successful commit. Or still someone might
 want to launch asynchronous code after.  A post-commit hook is
 available for such use cases: use addAfterCommitHook(), passing it a
@@ -64,7 +64,7 @@ callable and arguments.  The callable will be called with a Boolean
 value representing the status of the commit operation as first
 argument (true if successfull or false iff aborted) preceding its
 arguments at the start of the commit (but not for substransaction
-commits).
+commits). Commit hooks are not called for transaction.abort().
 
 Error handling
 --------------
@@ -97,16 +97,19 @@ committed or aborted.  The methods are passed the current Transaction
 as their only argument.
 """
 
+import binascii
 import logging
 import sys
-import thread
-import warnings
 import weakref
 import traceback
-from cStringIO import StringIO
 
 from zope import interface
 
+from transaction.compat import reraise
+from transaction.compat import get_thread_ident
+from transaction.compat import native_
+from transaction.compat import bytes_
+from transaction.compat import StringIO
 from transaction.weakset import WeakSet
 from transaction.interfaces import TransactionFailedError
 from transaction import interfaces
@@ -174,7 +177,7 @@ class Transaction(object):
         # directly by storages, leading underscore notwithstanding.
         self._extension = {}
 
-        self.log = logging.getLogger("txn.%d" % thread.get_ident())
+        self.log = logging.getLogger("txn.%d" % get_thread_ident())
         self.log.debug("new transaction")
 
         # If a commit fails, the traceback is saved in _failure_traceback.
@@ -242,6 +245,13 @@ class Transaction(object):
                 transaction_savepoint._savepoints.append(
                     datamanager_savepoint)
 
+    def _unjoin(self, resource):
+        # Leave a transaction because a savepoint was rolled back on a resource
+        # that joined later.
+
+        # Don't use remove.  We don't want to assume anything about __eq__.
+        self._resources = [r for r in self._resources if r is not resource]
+
     def savepoint(self, optimistic=False):
         if self.status is Status.COMMITFAILED:
             self._prior_operation_failed() # doesn't return, it raises
@@ -265,8 +275,8 @@ class Transaction(object):
     def _remove_and_invalidate_after(self, savepoint):
         savepoint2index = self._savepoint2index
         index = savepoint2index[savepoint]
-        # use items() to make copy to avoid mutating while iterating
-        for savepoint, i in savepoint2index.items():
+        # use list(items()) to make copy to avoid mutating while iterating
+        for savepoint, i in list(savepoint2index.items()):
             if i > index:
                 savepoint.transaction = None # invalidate
                 del savepoint2index[savepoint]
@@ -305,7 +315,8 @@ class Transaction(object):
 
     def commit(self):
         if self.status is Status.DOOMED:
-            raise interfaces.DoomedTransaction()
+            raise interfaces.DoomedTransaction(
+                'transaction doomed, cannot commit')
 
         if self._savepoint2index:
             self._invalidate_all_savepoints()
@@ -322,9 +333,15 @@ class Transaction(object):
             self._commitResources()
             self.status = Status.COMMITTED
         except:
-            t, v, tb = self._saveAndGetCommitishError()
-            self._callAfterCommitHooks(status=False)
-            raise t, v, tb
+            t = None
+            v = None
+            tb = None
+            try:
+                t, v, tb = self._saveAndGetCommitishError()
+                self._callAfterCommitHooks(status=False)
+                reraise(t, v, tb)
+            finally:
+                del t, v, tb
         else:
             if self._manager:
                 self._manager.free(self)
@@ -336,18 +353,32 @@ class Transaction(object):
         self.status = Status.COMMITFAILED
         # Save the traceback for TransactionFailedError.
         ft = self._failure_traceback = StringIO()
-        t, v, tb = sys.exc_info()
-        # Record how we got into commit().
-        traceback.print_stack(sys._getframe(1), None, ft)
-        # Append the stack entries from here down to the exception.
-        traceback.print_tb(tb, None, ft)
-        # Append the exception type and value.
-        ft.writelines(traceback.format_exception_only(t, v))
-        return t, v, tb
+        t = None
+        v = None
+        tb = None
+        try:
+            t, v, tb = sys.exc_info()
+            # Record how we got into commit().
+            traceback.print_stack(sys._getframe(1), None, ft)
+            # Append the stack entries from here down to the exception.
+            traceback.print_tb(tb, None, ft)
+            # Append the exception type and value.
+            ft.writelines(traceback.format_exception_only(t, v))
+            return t, v, tb
+        finally:
+            del t, v, tb
+        
 
     def _saveAndRaiseCommitishError(self):
-        t, v, tb = self._saveAndGetCommitishError()
-        raise t, v, tb
+        t = None
+        v = None
+        tb = None
+        try:
+            t, v, tb = self._saveAndGetCommitishError()
+            reraise(t, v, tb)
+        finally:
+            del t, v, tb
+            
 
     def getBeforeCommitHooks(self):
         return iter(self._before_commit)
@@ -408,7 +439,7 @@ class Transaction(object):
         # Execute the two-phase commit protocol.
 
         L = list(self._resources)
-        L.sort(rm_cmp)
+        L.sort(key=rm_key)
         try:
             for rm in L:
                 rm.tpc_begin(self)
@@ -435,10 +466,13 @@ class Transaction(object):
             # to revert the changes in each of the resource managers.
             t, v, tb = sys.exc_info()
             try:
-                self._cleanup(L)
+                try:
+                    self._cleanup(L)
+                finally:
+                    self._synchronizers.map(lambda s: s.afterCompletion(self))
+                reraise(t, v, tb)
             finally:
-                self._synchronizers.map(lambda s: s.afterCompletion(self))
-            raise t, v, tb
+                del t, v, tb
 
     def _cleanup(self, L):
         # Called when an exception occurs during tpc_vote or tpc_finish.
@@ -462,30 +496,37 @@ class Transaction(object):
 
         self._synchronizers.map(lambda s: s.beforeCompletion(self))
 
-        tb = None
-        for rm in self._resources:
-            try:
-                rm.abort(self)
-            except:
-                if tb is None:
-                    t, v, tb = sys.exc_info()
-                self.log.error("Failed to abort resource manager: %s",
-                               rm, exc_info=sys.exc_info())
+        try:
 
-        if self._manager:
-            self._manager.free(self)
+            t = None
+            v = None
+            tb = None
+            
+            for rm in self._resources:
+                try:
+                    rm.abort(self)
+                except:
+                    if tb is None:
+                        t, v, tb = sys.exc_info()
+                    self.log.error("Failed to abort resource manager: %s",
+                                   rm, exc_info=sys.exc_info())
 
-        self._synchronizers.map(lambda s: s.afterCompletion(self))
+            if self._manager:
+                self._manager.free(self)
 
-        self.log.debug("abort")
+            self._synchronizers.map(lambda s: s.afterCompletion(self))
 
-        if tb is not None:
-            raise t, v, tb
+            self.log.debug("abort")
+
+            if tb is not None:
+                reraise(t, v, tb)
+        finally:
+            del t, v, tb
 
     def note(self, text):
         text = text.strip()
         if self.description:
-            self.description += "\n\n" + text
+            self.description += "\n" + text
         else:
             self.description = text
 
@@ -535,22 +576,30 @@ class MultiObjectResourceAdapter(object):
         self.manager.tpc_vote(txn)
 
     def abort(self, txn):
+        t = None
+        v = None
         tb = None
-        for o in self.objects:
-            try:
-                self.manager.abort(o, txn)
-            except:
-                # Capture the first exception and re-raise it after
-                # aborting all the other objects.
-                if tb is None:
-                    t, v, tb = sys.exc_info()
-                txn.log.error("Failed to abort object: %s",
-                              object_hint(o), exc_info=sys.exc_info())
-        if tb is not None:
-            raise t, v, tb
+        try:
+            for o in self.objects:
+                try:
+                    self.manager.abort(o, txn)
+                except:
+                    # Capture the first exception and re-raise it after
+                    # aborting all the other objects.
+                    if tb is None:
+                        t, v, tb = sys.exc_info()
+                    txn.log.error("Failed to abort object: %s",
+                                  object_hint(o), exc_info=sys.exc_info())
 
-def rm_cmp(rm1, rm2):
-    return cmp(rm1.sortKey(), rm2.sortKey())
+            if tb is not None:
+                reraise(t, v, tb)
+        finally:
+            del t, v, tb
+
+def rm_key(rm):
+    func = getattr(rm, 'sortKey', None)
+    if func is not None:
+        return func()
 
 def object_hint(o):
     """Return a string describing the object.
@@ -569,7 +618,8 @@ def object_hint(o):
 def oid_repr(oid):
     if isinstance(oid, str) and len(oid) == 8:
         # Convert to hex and strip leading zeroes.
-        as_hex = hexlify(oid).lstrip('0')
+        as_hex = native_(
+            binascii.hexlify(bytes_(oid, 'ascii')), 'ascii').lstrip('0')
         # Ensure two characters per input byte.
         if len(as_hex) & 1:
             as_hex = '0' + as_hex
@@ -651,7 +701,8 @@ class Savepoint:
     def rollback(self):
         transaction = self.transaction
         if transaction is None:
-            raise interfaces.InvalidSavepointRollbackError
+            raise interfaces.InvalidSavepointRollbackError(
+                'invalidated by a later savepoint')
         transaction._remove_and_invalidate_after(self)
 
         try:
@@ -669,6 +720,7 @@ class AbortSavepoint:
 
     def rollback(self):
         self.datamanager.abort(self.transaction)
+        self.transaction._unjoin(self.datamanager)
 
 class NoRollbackSavepoint:
 
