@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2001, 2002, 2005 Zope Corporation and Contributors.
+# Copyright (c) 2001, 2002, 2005 Zope Foundation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -35,16 +35,13 @@ TODO
 
     add in tests for objects which are modified multiple times,
     for example an object that gets modified in multiple sub txns.
-
-$Id: test_transaction.py 81644 2007-11-09 16:27:32Z chrism $
 """
+from doctest import DocTestSuite, DocFileSuite
 
 import struct
+import sys
 import unittest
-import warnings
-
 import transaction
-from transaction.tests.warnhook import WarningsHook
 
 _ADDRESS_MASK = 256 ** struct.calcsize('P')
 def positive_id(obj):
@@ -94,7 +91,7 @@ class TransactionTests(unittest.TestCase):
         t.note('This is a note.')
         self.assertEqual(t.description, 'This is a note.')
         t.note('Another.')
-        self.assertEqual(t.description, 'This is a note.\n\nAnother.')
+        self.assertEqual(t.description, 'This is a note.\nAnother.')
 
         t.abort()
 
@@ -245,6 +242,25 @@ class TransactionTests(unittest.TestCase):
 ##            self.fail("Hosed Application didn't stop commits")
 
 
+class Test_oid_repr(unittest.TestCase):
+    def _callFUT(self, oid):
+        from transaction._transaction import oid_repr
+        return oid_repr(oid)
+
+    def test_as_nonstring(self):
+        self.assertEqual(self._callFUT(123), '123')
+
+    def test_as_string_not_8_chars(self):
+        self.assertEqual(self._callFUT('a'), "'a'")
+
+    def test_as_string_z64(self):
+        s = '\0'*8
+        self.assertEqual(self._callFUT(s), '0x00')
+
+    def test_as_string_all_Fs(self):
+        s = '\1'*8
+        self.assertEqual(self._callFUT(s), '0x0101010101010101')
+
 class DataObject:
 
     def __init__(self, transaction_manager, nost=0):
@@ -291,7 +307,7 @@ class BasicJar:
 
     def check(self, method):
         if self.tracing:
-            print '%s calling method %s'%(str(self.tracing),method)
+            print('%s calling method %s'%(str(self.tracing),method))
 
         if method in self.errors:
             raise TestTxnException("error %s" % method)
@@ -386,13 +402,14 @@ def test_addBeforeCommitHook():
 
     Now register the hook with a transaction.
 
+      >>> from transaction.compat import func_name
       >>> import transaction
       >>> t = transaction.begin()
       >>> t.addBeforeCommitHook(hook, '1')
 
     We can see that the hook is indeed registered.
 
-      >>> [(hook.func_name, args, kws)
+      >>> [(func_name(hook), args, kws)
       ...  for hook, args, kws in t.getBeforeCommitHooks()]
       [('hook', ('1',), {})]
 
@@ -446,7 +463,7 @@ def test_addBeforeCommitHook():
       ...     pass
       >>> class FailingDataManager:
       ...     def tpc_begin(self, txn, sub=False):
-      ...         raise CommitFailure
+      ...         raise CommitFailure('failed')
       ...     def abort(self, txn):
       ...         pass
 
@@ -454,10 +471,10 @@ def test_addBeforeCommitHook():
       >>> t.join(FailingDataManager())
 
       >>> t.addBeforeCommitHook(hook, '2')
-      >>> t.commit()
+      >>> t.commit() #doctest: +IGNORE_EXCEPTION_DETAIL
       Traceback (most recent call last):
       ...
-      CommitFailure
+      CommitFailure: failed
       >>> log
       ["arg '2' kw1 'no_kw1' kw2 'no_kw2'"]
       >>> reset_log()
@@ -470,7 +487,7 @@ def test_addBeforeCommitHook():
 
     They are returned in the same order by getBeforeCommitHooks.
 
-      >>> [(hook.func_name, args, kws)     #doctest: +NORMALIZE_WHITESPACE
+      >>> [(func_name(hook), args, kws)  #doctest: +NORMALIZE_WHITESPACE
       ...  for hook, args, kws in t.getBeforeCommitHooks()]
       [('hook', ('4',), {'kw1': '4.1'}),
        ('hook', ('5',), {'kw2': '5.2'})]
@@ -523,13 +540,14 @@ def test_addAfterCommitHook():
 
     Now register the hook with a transaction.
 
+      >>> from transaction.compat import func_name
       >>> import transaction
       >>> t = transaction.begin()
       >>> t.addAfterCommitHook(hook, '1')
 
     We can see that the hook is indeed registered.
 
-      >>> [(hook.func_name, args, kws)
+      >>> [(func_name(hook), args, kws)
       ...  for hook, args, kws in t.getAfterCommitHooks()]
       [('hook', ('1',), {})]
 
@@ -583,7 +601,7 @@ def test_addAfterCommitHook():
       ...     pass
       >>> class FailingDataManager:
       ...     def tpc_begin(self, txn):
-      ...         raise CommitFailure
+      ...         raise CommitFailure('failed')
       ...     def abort(self, txn):
       ...         pass
 
@@ -591,10 +609,10 @@ def test_addAfterCommitHook():
       >>> t.join(FailingDataManager())
 
       >>> t.addAfterCommitHook(hook, '2')
-      >>> t.commit()
+      >>> t.commit() #doctest: +IGNORE_EXCEPTION_DETAIL
       Traceback (most recent call last):
       ...
-      CommitFailure
+      CommitFailure: failed
       >>> log
       ["False arg '2' kw1 'no_kw1' kw2 'no_kw2'"]
       >>> reset_log()
@@ -607,7 +625,7 @@ def test_addAfterCommitHook():
 
     They are returned in the same order by getAfterCommitHooks.
 
-      >>> [(hook.func_name, args, kws)     #doctest: +NORMALIZE_WHITESPACE
+      >>> [(func_name(hook), args, kws)     #doctest: +NORMALIZE_WHITESPACE
       ...  for hook, args, kws in t.getAfterCommitHooks()]
       [('hook', ('4',), {'kw1': '4.1'}),
        ('hook', ('5',), {'kw2': '5.2'})]
@@ -673,8 +691,8 @@ def test_addAfterCommitHook():
       >>> do = DataObject(mgr)
 
       >>> t = transaction.begin()
-      >>> len(t._manager._txns)
-      1
+      >>> t._manager._txn is not None
+      True
 
       >>> t.addAfterCommitHook(hook, ('-', 1))
       >>> transaction.commit()
@@ -682,19 +700,78 @@ def test_addAfterCommitHook():
       >>> log
       ["True arg '-' kw1 1 kw2 'no_kw2'"]
 
-      >>> len(t._manager._txns)
-      0
+      >>> t._manager._txn is not None
+      False
 
       >>> reset_log()
     """
 
+def bug239086():
+    """
+    The original implementation of thread transaction manager made
+    invalid assumptions about thread ids.
+
+    >>> import transaction.tests.savepointsample
+    >>> dm = transaction.tests.savepointsample.SampleSavepointDataManager()
+    >>> list(dm.keys())
+    []
+
+    >>> class Sync:
+    ...      def __init__(self, label):
+    ...          self.label = label
+    ...      def beforeCompletion(self, t):
+    ...          print('%s %s' % (self.label, 'before'))
+    ...      def afterCompletion(self, t):
+    ...          print('%s %s' % (self.label, 'after'))
+    ...      def newTransaction(self, t):
+    ...          print('%s %s' % (self.label, 'new'))
+    >>> sync = Sync(1)
+
+    >>> import threading
+    >>> def run_in_thread(f):
+    ...     t = threading.Thread(target=f)
+    ...     t.start()
+    ...     t.join()
+
+    >>> @run_in_thread
+    ... def first():
+    ...     transaction.manager.registerSynch(sync)
+    ...     transaction.manager.begin()
+    ...     dm['a'] = 1
+    1 new
+
+    >>> @run_in_thread
+    ... def second():
+    ...     transaction.abort() # should do nothing.
+
+    >>> list(dm.keys())
+    ['a']
+
+    >>> dm = transaction.tests.savepointsample.SampleSavepointDataManager()
+    >>> list(dm.keys())
+    []
+
+    >>> @run_in_thread
+    ... def first():
+    ...     dm['a'] = 1
+
+    >>> transaction.abort() # should do nothing
+    >>> list(dm.keys())
+    ['a']
+
+    """
+
 def test_suite():
-    from zope.testing.doctest import DocTestSuite, DocFileSuite
-    return unittest.TestSuite((
+    suite = unittest.TestSuite((
         DocFileSuite('doom.txt'),
         DocTestSuite(),
         unittest.makeSuite(TransactionTests),
+        unittest.makeSuite(Test_oid_repr),
         ))
+    if sys.version_info >= (2, 6):
+        suite.addTest(DocFileSuite('convenience.txt'))
+
+    return suite
 
 # additional_tests is for setuptools "setup.py test" support
 additional_tests = test_suite
